@@ -19,7 +19,8 @@ from vantage6.common import bytes_to_base64s, base64s_to_bytes
 from vantage6.common.globals import APPNAME
 from vantage6.client import serialization, deserialization
 from vantage6.client.filter import post_filtering
-from vantage6.client.encryption import RSACryptor, DummyCryptor
+from vantage6.client.encryption import CryptorBase, RSACryptor, DummyCryptor
+from vantage6.client.exceptions import DeserializationException
 
 
 module_name = __name__.split('.')[1]
@@ -148,7 +149,7 @@ class ClientBase(object):
         return path
 
     def request(self, endpoint: str, json: dict = None, method: str = 'get',
-                params: dict=None, first_try: bool=True) -> dict:
+                params: dict = None, first_try: bool = True) -> dict:
         """Create http(s) request to the vantage6 server
 
         Parameters
@@ -167,7 +168,7 @@ class ClientBase(object):
         Returns
         -------
         dict
-            Eesponse of the server
+            Response of the server
         """
 
         # get appropiate method
@@ -348,7 +349,7 @@ class ClientBase(object):
     def post_task(self, name: str, image: str, collaboration_id: int,
                   input_='', description='',
                   organization_ids: list = None,
-                  data_format=LEGACY, database: str='default') -> dict:
+                  data_format=LEGACY) -> dict:
         """Post a new task at the server
 
         It will also encrypt `input_` for each receiving organization.
@@ -401,7 +402,8 @@ class ClientBase(object):
 
             organization_json_list.append({
                 "id": org_id,
-                "input": self.cryptor.encrypt_bytes_to_str(serialized_input, pub_key)
+                "input": self.cryptor.encrypt_bytes_to_str(serialized_input,
+                                                           pub_key)
             })
 
         return self.request('task', method='post', json={
@@ -409,13 +411,12 @@ class ClientBase(object):
             "image": image,
             "collaboration_id": collaboration_id,
             "description": description,
-            "organizations": organization_json_list,
-            'database': database
+            "organizations": organization_json_list
         })
 
-    def get_results(self, id: int=None, state: str=None,
-                    include_task: bool=False, task_id: int=None,
-                    node_id: int=None) -> dict:
+    def get_results(self, id: int = None, state: str = None,
+                    include_task: bool = False, task_id: int = None,
+                    node_id: int = None) -> dict:
         """Get task result(s) from the central server
 
         Depending if a `id` is specified or not, either a single or a
@@ -444,33 +445,6 @@ class ClientBase(object):
         dict
             Containing the result(s)
         """
-        def decrypt_result(res):
-            """Helper to decrypt the keys 'input' and 'result' in dict.
-
-            Keys are replaced, but object reference remains intact: changes are
-            made *in-place*.
-            """
-            cryptor = self.cryptor
-            try:
-                self.log.info('Decrypting input')
-                # TODO this only works when the results belong to the
-                # same organization... We should make different implementation
-                # of get_results
-                res["input"] = cryptor.decrypt_str_to_bytes(res["input"])
-
-            except Exception as e:
-                self.log.debug(e)
-
-            try:
-                if res["result"]:
-                    self.log.info('Decrypting result')
-                    res["result"] = cryptor.decrypt_str_to_bytes(res["result"])
-
-            except ValueError as e:
-                self.log.error("Could not decrypt/decode input or result.")
-                self.log.error(e)
-                # raise
-
         # Determine endpoint and create dict with query parameters
         endpoint = 'result' if not id else f'result/{id}'
         params = dict()
@@ -487,21 +461,54 @@ class ClientBase(object):
         # self.log.debug(f"Retrieving results using query parameters:{params}")
         results = self.request(endpoint=endpoint, params=params)
 
+        if isinstance(results, str):
+            self.log.warn("Requesting results failed")
+            self.log.debug(f"Results message: {results}")
+            return {}
+
         if id:
             # Single result
-            decrypt_result(results)
+            self._decrypt_result(results)
 
         else:
             # Multiple results
             for result in results:
-                decrypt_result(result)
+                self._decrypt_result(result)
 
         return results
 
+    def _decrypt_result(self, result):
+        """Helper to decrypt the keys 'input' and 'result' in dict.
+
+        Keys are replaced, but object reference remains intact: changes are
+        made *in-place*.
+        """
+        cryptor = self.cryptor
+        try:
+            self.log.info('Decrypting input')
+            # TODO this only works when the results belong to the
+            # same organization... We should make different implementation
+            # of get_results
+            result["input"] = cryptor.decrypt_str_to_bytes(result["input"])
+
+        except Exception as e:
+            self.log.debug(e)
+
+        try:
+            if result["result"]:
+                self.log.info('Decrypting result')
+                result["result"] = \
+                    cryptor.decrypt_str_to_bytes(result["result"])
+
+        except ValueError as e:
+            self.log.error("Could not decrypt/decode input or result.")
+            self.log.error(e)
+            # raise
+
     class SubClient:
         """Create sub groups of commands using this SubClient"""
         def __init__(self, parent):
-                self.parent = parent
+            self.parent = parent
 
 
 class UserClient(ClientBase):
@@ -1288,7 +1295,7 @@ class UserClient(ClientBase):
         @post_filtering(iterable=False)
         def create(self, collaboration: int, organizations: list, name: str,
                    image: str, description: str, input: dict,
-                   data_format: str=LEGACY, database: str='default') -> dict:
+                   data_format: str=LEGACY) -> dict:
             """Create a new task
 
             Parameters
@@ -1308,19 +1315,15 @@ class UserClient(ClientBase):
                 Algorithm input
             data_format : str, optional
                 IO data format used, by default LEGACY
-            database: str, optional
-                Name of the database to use. This should match the key
-                in the node configuration files. If not specified the
-                default database will be tried.
 
             Returns
             -------
             dict
-                Containing the task information
+                [description]
             """
             return self.parent.post_task(name, image, collaboration, input,
                                          description, organizations,
-                                         data_format, database)
+                                         data_format)
 
         def delete(self, id_: int) -> dict:
             """Delete a task
@@ -1450,7 +1453,7 @@ class UserClient(ClientBase):
 class ContainerClient(ClientBase):
     """ Container interface to the local proxy server (central server).
 
-        A algorithm container (should) never communicate directly to the
+        An algorithm container should never communicate directly to the
         central server. Therefore the algorithm container has no
         internet connection. The algorithm can, however, talk to a local
         proxy server which has interface to the central server. This way
@@ -1463,8 +1466,8 @@ class ContainerClient(ClientBase):
 
     def __init__(self, token: str, *args, **kwargs):
         """Container client.
-        A client which can be used by algorithms. All permissions of the container are
-        derived from the token.
+        A client which can be used by algorithms. All permissions of the
+        container are derived from the token.
 
         Parameters
         ----------
@@ -1477,7 +1480,6 @@ class ContainerClient(ClientBase):
         # obtain the identity from the token
         container_identity = jwt.decode(token, verify=False)['identity']
         self.image = container_identity.get("image")
-        self.database = container_identity.get('database')
         self.host_node_id = container_identity.get("node_id")
         self.collaboration_id = container_identity.get("collaboration_id")
         self.log.info(
@@ -1516,11 +1518,36 @@ class ContainerClient(ClientBase):
             f"task/{task_id}/result"
         )
 
-        res = [pickle.loads(base64s_to_bytes(result.get("result")))
-               for result in results]
+        res = []
+        # Encryption is not done at the client level for the container.
+        # Although I am not completely sure that the format is always
+        # a pickle.
+        # for result in results:
+        #     self._decrypt_result(result)
+        #     res.append(result.get("result"))
+        #
+        try:
+            res = [pickle.loads(base64s_to_bytes(result.get("result")))
+                   for result in results if result.get("result")]
+        except Exception as e:
+            self.log.error('Unable to unpickle result')
+            self.log.debug(e)
 
         return res
 
+    def get_other_node_ip_and_port(self, task_id: int):
+        """
+        Return IP address and port number of other nodes involved in a task
+        so that VPN can be used for communication
+        """
+        results = self.request(f"task/{task_id}/result")
+
+        res = []
+        for result in results:
+            d = {'ip' : result['node']['ip'], 'port': result['port']}
+            res.append(d)
+        return res
+
     def get_task(self, task_id: int):
         return self.request(
             f"task/{task_id}"
@@ -1545,8 +1572,7 @@ class ContainerClient(ClientBase):
             collaboration_id=self.collaboration_id,
             organization_ids=organization_ids,
             input_=input_,
-            image=self.image,
-            database=self.database
+            image=self.image
         )
 
     def get_organizations_in_my_collaboration(self):
@@ -1563,7 +1589,7 @@ class ContainerClient(ClientBase):
 
     def post_task(self, name: str, image: str, collaboration_id: int,
                   input_: str = '', description='',
-                  organization_ids: list = [], database='default') -> dict:
+                  organization_ids: list = []) -> dict:
         """ Post a new task at the central server.
 
             ! To create a new task from the algorithm container you
@@ -1603,8 +1629,7 @@ class ContainerClient(ClientBase):
             "image": image,
             "collaboration_id": collaboration_id,
             "description": description,
-            "organizations": organization_json_list,
-            "database": database
+            "organizations": organization_json_list
         })
 
 
